@@ -1,4 +1,4 @@
-/* $Id: array.h 2553 2011-07-09 09:42:33Z tcunha $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -39,10 +39,10 @@
 		fatalx("size too big");					\
 	if ((a)->space == 0) {						\
 	       	(a)->space = ARRAY_INITIALSPACE(a);			\
-		(a)->list = xrealloc((a)->list, 1, (a)->space);		\
+		(a)->list = xrealloc((a)->list, (a)->space);		\
 	}								\
 	while ((a)->space <= ((a)->num + (n)) * ARRAY_ITEMSIZE(a)) {	\
-		(a)->list = xrealloc((a)->list, 2, (a)->space);		\
+		(a)->list = xreallocarray((a)->list, 2, (a)->space);	\
 		(a)->space *= 2;					\
 	}								\
 } while (0)
@@ -109,13 +109,12 @@
 } while (0)
 
 #define ARRAY_FREE(a) do {						\
-	if ((a)->list != NULL)						\
-		xfree((a)->list);					\
+	free((a)->list);						\
 	ARRAY_INIT(a);							\
 } while (0)
 #define ARRAY_FREEALL(a) do {						\
 	ARRAY_FREE(a);							\
-	xfree(a);							\
+	free(a);							\
 } while (0)
 
 #endif
