@@ -1,4 +1,4 @@
-/* $Id$ */
+/* $OpenBSD$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -26,21 +26,29 @@
  * List key bindings.
  */
 
-enum cmd_retval	 cmd_list_keys_exec(struct cmd *, struct cmd_ctx *);
-enum cmd_retval	 cmd_list_keys_table(struct cmd *, struct cmd_ctx *);
+enum cmd_retval	 cmd_list_keys_exec(struct cmd *, struct cmd_q *);
+
+enum cmd_retval	 cmd_list_keys_table(struct cmd *, struct cmd_q *);
+enum cmd_retval	 cmd_list_keys_commands(struct cmd *, struct cmd_q *);
 
 const struct cmd_entry cmd_list_keys_entry = {
 	"list-keys", "lsk",
 	"t:", 0, 0,
 	"[-t key-table]",
 	0,
-	NULL,
-	NULL,
+	cmd_list_keys_exec
+};
+
+const struct cmd_entry cmd_list_commands_entry = {
+	"list-commands", "lscm",
+	"", 0, 0,
+	"",
+	0,
 	cmd_list_keys_exec
 };
 
 enum cmd_retval
-cmd_list_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
+cmd_list_keys_exec(struct cmd *self, struct cmd_q *cmdq)
 {
 	struct args		*args = self->args;
 	struct key_binding	*bd;
@@ -49,8 +57,11 @@ cmd_list_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 	size_t			 used;
 	int			 width, keywidth;
 
+	if (self->entry == &cmd_list_commands_entry)
+		return (cmd_list_keys_commands(self, cmdq));
+
 	if (args_has(args, 't'))
-		return (cmd_list_keys_table(self, ctx));
+		return (cmd_list_keys_table(self, cmdq));
 
 	width = 0;
 
@@ -91,14 +102,14 @@ cmd_list_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 			continue;
 
 		cmd_list_print(bd->cmdlist, tmp + used, (sizeof tmp) - used);
-		ctx->print(ctx, "bind-key %s", tmp);
+		cmdq_print(cmdq, "bind-key %s", tmp);
 	}
 
 	return (CMD_RETURN_NORMAL);
 }
 
-int
-cmd_list_keys_table(struct cmd *self, struct cmd_ctx *ctx)
+enum cmd_retval
+cmd_list_keys_table(struct cmd *self, struct cmd_q *cmdq)
 {
 	struct args			*args = self->args;
 	const char			*tablename;
@@ -109,7 +120,7 @@ cmd_list_keys_table(struct cmd *self, struct cmd_ctx *ctx)
 
 	tablename = args_get(args, 't');
 	if ((mtab = mode_key_findtable(tablename)) == NULL) {
-		ctx->error(ctx, "unknown key table: %s", tablename);
+		cmdq_error(cmdq, "unknown key table: %s", tablename);
 		return (CMD_RETURN_ERROR);
 	}
 
@@ -138,11 +149,33 @@ cmd_list_keys_table(struct cmd *self, struct cmd_ctx *ctx)
 			mode = "c";
 		cmdstr = mode_key_tostring(mtab->cmdstr, mbind->cmd);
 		if (cmdstr != NULL) {
-			ctx->print(ctx, "bind-key -%st %s%s %*s %s",
+			cmdq_print(cmdq, "bind-key -%st %s%s %*s %s%s%s%s",
 			    mode, any_mode && *mode == '\0' ? " " : "",
-			    mtab->name, (int) width, key, cmdstr);
+			    mtab->name, (int) width, key, cmdstr,
+			    mbind->arg != NULL ? " \"" : "",
+			    mbind->arg != NULL ? mbind->arg : "",
+			    mbind->arg != NULL ? "\"": "");
 		}
 	}
 
 	return (CMD_RETURN_NORMAL);
 }
+
+enum cmd_retval
+cmd_list_keys_commands(unused struct cmd *self, struct cmd_q *cmdq)
+{
+	const struct cmd_entry	**entryp;
+	const struct cmd_entry	 *entry;
+
+	for (entryp = cmd_table; *entryp != NULL; entryp++) {
+		entry = *entryp;
+		if (entry->alias == NULL) {
+			cmdq_print(cmdq, "%s %s", entry->name, entry->usage);
+			continue;
+		}
+		cmdq_print(cmdq, "%s (%s) %s", entry->name, entry->alias,
+		    entry->usage);
+	}
+
+	return (CMD_RETURN_NORMAL);
+}
